@@ -1,13 +1,13 @@
 use strict;
 package Devel::Caller;
 use warnings;
-use B;
+use B qw( peekop );
 use PadWalker ();
 use XSLoader;
 use base qw( Exporter  );
 use 5.008;
 
-our $VERSION = '2.05';
+our $VERSION = '2.07';
 XSLoader::load __PACKAGE__, $VERSION;
 
 our @EXPORT_OK = qw( caller_cv caller_args caller_vars called_with called_as_method );
@@ -21,20 +21,23 @@ sub caller_cv {
 
 our $DEBUG = 0;
 
+# scan forward through the ops noting the pushmark or a padrange ops.
+# These indicate the start of a subroutine call.  We're looking for the most
+# recent one before the subroutine invocation (the entersub).
 sub scan_forward {
     my $op = shift;
-    die "was expecting a pushmark, not a " . $op->name
-      if ($op->name ne "pushmark");
+    die "was expecting a pushmark or a padrange, not a " . $op->name
+      if ($op->name !~ /^(?:pushmark|padrange)$/);
 
     my @stack;
     for (; $op && $op->name ne 'entersub'; $op = $op->next) {
-        print "SCAN op $op ", $op->name, "\n" if $DEBUG;
-        if ($op->name eq "pushmark") {
-            print "push $op\n" if $DEBUG;
+        print "SCAN ", peekop($op), "\n" if $DEBUG;
+        if ($op->name eq "pushmark" or $op->name eq "padrange") {
+            print "   PUSH\n" if $DEBUG;
             push @stack, $op;
         }
         elsif (0) { # op consumes a mark
-            print "pop\n" if $DEBUG;
+            print "   POP\n" if $DEBUG;
             pop @stack;
         }
     }
@@ -52,42 +55,70 @@ sub called_with {
     my $padn = $pad->ARRAYelt( 0 );
     my $padv = $pad->ARRAYelt( 1 );
 
-    print $op->name, "\n" if $DEBUG;
+    print "Context OP: ", peekop($op), "\n" if $DEBUG;
     $op = scan_forward( $op );
-    print $op->name, "\n" if $DEBUG;
+    print "Scanned forward to ", peekop($op), "\n" if $DEBUG;
 
     my @return;
-    my ($prev, $skip);
-    $skip = 0;
-    while (($prev = $op) && ($op = $op->next) && ($op->name ne "entersub")) {
-        print "op $op ", $op->name, "\n" if $DEBUG;
-        if ($op->name eq "pushmark") {
-            $skip = !$skip;
+    my $prev;
+
+    # We're scanning through looking for ops which are pushing
+    # variables onto the stack (/pad(sv|av|hv)/ push from the pad, 
+    # /gvsv|rv2([ahg]v/ are from globs.
+    for (; $op && $op->name ne 'entersub'; ($prev = $op) && ($op = $op->next)) {
+        printf "Loop: %s %s targ: %d\n", peekop($op), $op->name, $op->targ if $DEBUG;
+
+        if ($op->name eq "padrange") {
+            # A padrange is a 5.17 optimisation that uses a single op to
+            # load multiple pad variables onto the stack.  The old ops
+            # are preserved and are reachable as the padrange's sibling
+            # so that B::Deparse can pessimise it back to that state.
+            #
+            # http://perl5.git.perl.org/perl.git/commitdiff/0fe870f5
+            # http://perl5.git.perl.org/perl.git/commitdiff/a7fd8ef6
+            #
+            # We could use the B::Deparse method, but it's probably simpler if
+            # we just reassign $op.
+            print "padrange, diverting down ", $prev->sibling, "\n" if $DEBUG;
+            $op = $op->sibling;
+        }
+
+        if ($op->name =~ /padsv_store/) {
+            # A padsv_store is a 5.37 optimization that combines a padsv and
+            # an sassign into a single op. The new op steals the targ slot
+            # of the original padsv.
+            #
+            # https://github.com/Perl/perl5/commit/9fdd7fc
+            print "Copying from pad\n" if $DEBUG;
+            if ($want_names) {
+                push @return, $padn->ARRAYelt( $op->targ )->PVX;
+            }
+            else {
+                push @return, $padv->ARRAYelt( $op->targ )->object_2svref;
+            }
+            next;
         }
         elsif ($op->name =~ "pad(sv|av|hv)") {
-            next if $skip;
-            print "PAD skip:$skip\n" if $DEBUG;
-
             if ($op->next->next->name eq "sassign") {
-                $skip = 0;
+                print "sassign in two ops, this is the target skipping\n" if $DEBUG;
+                next;
+            } elsif ($op->next->name eq "padsv_store") {
+                print "padsv_store in one op, this is the target, skipping\n" if $DEBUG;
                 next;
             }
 
-            print "targ: ", $op->targ, "\n" if $DEBUG;
-            my $name  = $padn->ARRAYelt( $op->targ )->PVX;
-            my $value = $padv->ARRAYelt( $op->targ )->object_2svref;
-            push @return, $want_names ? $name : $value;
-            next;
-        }
-        elsif ($op->name eq "gv") {
+            print "Copying from pad\n" if $DEBUG;
+            if ($want_names) {
+                push @return, $padn->ARRAYelt( $op->targ )->PVX;
+            }
+            else {
+                push @return, $padv->ARRAYelt( $op->targ )->object_2svref;
+            }
             next;
         }
         elsif ($op->name =~ /gvsv|rv2(av|hv|gv)/) {
-            print "GV skip:$skip\n" if $DEBUG;
-
             if ($op->next->next->name eq "sassign") {
-                $skip = 0;
-                print "skipped\n" if $DEBUG;
+                print "sassign in two ops, this is the target, skipping\n" if $DEBUG;
                 next;
             }
 
@@ -129,9 +160,11 @@ sub called_with {
             next;
         }
         elsif ($op->name eq "const") {
-            print "const $op skip:$skip\n" if $DEBUG;
             if ($op->next->next->name eq "sassign") {
-                $skip = 0;
+                print "sassign in two ops, this is the target, skipping\n" if $DEBUG;
+                next;
+            } elsif ($op->next->name eq "padsv_store") {
+                print "padsv_store in one op, this is the target, skipping\n" if $DEBUG;
                 next;
             }
 
@@ -148,7 +181,7 @@ sub called_as_method {
     my $op = _context_op( PadWalker::_upcontext( $level + 1 ));
 
     print "called_as_method: $op\n" if $DEBUG;
-    die "was expecting a pushmark, not a ". $op->name
+    die "was expecting a pushmark or pad, not a ". $op->name
       unless $op->name eq "pushmark";
     while (($op = $op->next) && ($op->name ne "entersub")) {
         print "method: ", $op->name, "\n" if $DEBUG;
@@ -210,13 +243,14 @@ constants are returned as C<undef> in both cases
 C<called_as_method> returns true if the subroutine at $level was
 called as a method.
 
+=back
 
 =head1 BUGS
 
 All of these routines are susceptible to the same limitations as
 C<caller> as described in L<perlfunc/caller>
 
-The deparsing of the optree perfomed by called_with is fairly simple-minded
+The deparsing of the optree performed by called_with is fairly simple-minded
 and so a bit flaky.
 
 =over
@@ -240,10 +274,10 @@ PadWalker by Robin Houston
 
 =head1 COPYRIGHT
 
-Copyright (c) 2002, 2003, 2006, 2007, 2008, 2010 Richard Clamp. All Rights
-Reserved.
+Copyright (c) 2002, 2003, 2006, 2007, 2008, 2010, 2013, 2023 Richard Clamp.
+All Rights Reserved.
+
 This module is free software. It may be used, redistributed and/or
 modified under the same terms as Perl itself.
 
 =cut
-
